/* es-sinh.c */
#include <stdio.h>
#include <math.h>
#include <assert.h>

/* SPECIFICA DEL PROBLEMA
   ======================

   Realizzare la funzione my_sinh() come da specifica seguente.  La
   funzione calcola sinh(x) (seno iperbolico di x) usando l'espansione
   in serie. Si ricorda che lo sviluppo in Serie di Taylor-MacLaurain
   della funzione trigonometrica sinh(x) è calcolato come:

             ___+inf
             \       x^(2n+1)
   sinh(x) =  >     ----------- = x + (x^3)/3! + (x^5)/5! + ...
             /__     (2n + 1)!
                n=0

   nella formula precedente l'operatore ^ indica l'elevamento a
   potenza (a differenza del linguaggio C, in cui indica tutt'altro).
   Ricordare che (2n+1)! indica "(2n+1) fattoriale", cioe' il prodotto
   di tutti gli interi da 1 a (2n+1), estremi inclusi (per definizione
   si pone 0! = 1).

   La funzione my_sinh() seguente accetta come parametri il valore x,
   e un intero k>=0 che indica in quale punto troncare la serie
   precedente.  In altre parole, la funzione deve calcolare la
   sommatoria (finita)


                  ___k   
                  \       x^(2n+1)
   my_sinh(x,k) =  >     -----------
                  /__     (2n + 1)!
                     n=0

   Quindi, ad esempio:

   my_sinh(x,0) = x
   my_sinh(x,1) = x + (x^3)/3!
   my_sinh(x,2) = x + (x^3)/3! + (x^5)/5!
   ...

   La funzione main() confronta il risultato della funzione my_sinh
   con il risultato "esatto" calcolato dalla funzione sinh() definita
   in math.h (che viene gia' incluso in questo file), per valori
   crescenti di k. Si dovrebbe vedere che l'errore relativo tende a
   zero al crescere di k.

   Se lo si ritiene utile, si può usare la funzione 

   double pow(double x, double y)

   definita in math.h che calcola x elevato alla y.
*/

/* calcola il fattoriale di n */
int fatt(int n)
{
    int f = 1;
    for ( ; n>1; n-- ) {
        f = f * n;
    }
    return f;
}

/* calcola sinh(x) usando l'espansione in serie. Si veda
   https://en.wikipedia.org/wiki/Taylor_series */
double my_sinh(double x, int k)
{
    int n;
    double result = 0.0;
    for (n=0; n<=k; n++) {
        result += pow(x, 2*n+1) / fatt(2*n+1);
    }
    return result;
}

/* Versione piu' efficiente, ma meno leggibile. Calcoliamo ogni
   termine della sommatoria partendo dal precedente. Si noti che
   questa versione non usa né pow() né fatt() */
double my_sinh2(double x, int k)
{
    int n;
    double term = x; /* termine precedente della sommatoria */
    double result = term;
    for (n=1; n<=k; n++) {
        term = term * x * x / ((2*n+1) * 2*n);
        result += term;
    }
    return result;
}

int main( void )
{
    int k;
    const double x = 0.3;
    for (k=0; k<10; k++) {
        const double approx = my_sinh(x,k);
        const double exact = sinh(x);
        const double relerr = fabs(exact - approx) / fabs(exact);
        printf("sinh(%f): exact=%f approx=%f relerr=%g\n", x, exact, approx, relerr);
    }
    return 0;
}
